import os

from django import forms
from django.conf import settings
from django.contrib import admin, messages
from django.shortcuts import render_to_response
from django.template import Context, RequestContext, Template

from .models import FileUpload, ImageUpload, ThumbnailSize


class ImageUploadAdmin(admin.ModelAdmin):
    list_display = (
        'name',
        'display_image',
    )

    search_fields = (
        'name',
        'image',
    )

    actions = [
        'delete_selected',
    ]

    if 'danemco.gallery' in settings.INSTALLED_APPS:
        actions.append('send_to_gallery')

    def send_to_gallery(self, request, queryset):
        from danemco.gallery.models import Album, Photo

        class GalleryForm(forms.Form):
            album = forms.ModelChoiceField(Album.objects.all(), required=True)

        if 'album' in request.POST:
            form = GalleryForm(request.POST)
            if form.is_valid():
                for obj in queryset.all():
                    album = form.cleaned_data['album']
                    photo, created = Photo.objects.get_or_create(album=album, image=obj.image, defaults={'name': os.path.basename(obj.image.name)})
                messages.add_message(request, messages.INFO, 'Images sent to album "%s"' % (album,))
                return None
        else:
            form = GalleryForm()

        return render_to_response(
            'uploader/send_to_gallery.html',
            locals(),
            context_instance=RequestContext(request)
        )

    def display_image(self, obj):
        return Template(
            '{% load thumbnail %}'
            '<img src="{% thumbnail object.image "30x30" %}" alt>'
        ).render(Context({
            'object': obj,
        }))
    display_image.allow_tags = True
    display_image.admin_order_field = 'image'


class FileUploadAdmin(admin.ModelAdmin):
    list_display = (
        'name',
        'file',
    )

    search_fields = (
        'name',
        'file',
    )

admin.site.register(ImageUpload, ImageUploadAdmin)
admin.site.register(FileUpload, FileUploadAdmin)
admin.site.register(ThumbnailSize)



from django.db import models
from .utils import make_thumbnail

UL_PATH = 'uploads/%Y/%m/%d/'


class ThumbnailSize(models.Model):
    name = models.CharField(max_length=30)
    height = models.PositiveIntegerField()
    width = models.PositiveIntegerField()

    def as_size(self):
        return u"%sx%s" % (self.width, self.height)

    def __unicode__(self):
        return u'%s (%ix%i)' % (self.name, self.width, self.height)

    class Meta:
        unique_together = (('height', 'width'),)
        ordering = ('width', 'height')


class Thumbnail(models.Model):
    upload = models.ForeignKey('ImageUpload')
    size = models.ForeignKey(ThumbnailSize)
    image = models.ImageField(upload_to='%simages/' % UL_PATH, max_length=255)

    def __unicode__(self):
        try:
            return u'%s (%ix%i)' % (self.upload,
                                    self.image.width,
                                    self.image.height)
        except:
            return u'%s (%s)' % (self.upload, self.size.name)

    class Meta:
        unique_together = (('upload', 'size'),)


class ImageUpload(models.Model):
    name = models.CharField(max_length=50)
    image = models.ImageField(upload_to='%simages/' % UL_PATH, max_length=255)
    resize_image = models.ManyToManyField(ThumbnailSize, blank=True, help_text='Choose the size or sizes you would like the system to create for you. Choose none of the sizes to preserve the dimensions of the image you are uploading.')

    """
    def __init__(self, *args, **kwargs):
        super(ImageUpload, self).__init__(*args, **kwargs)

        if self.id and not self.original_name:
            self.original_name = self.image.name
            self.save()
    """

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return self.image.url


class FileUpload(models.Model):
    name = models.CharField(max_length=50)
    file = models.FileField(upload_to='%sfiles/' % UL_PATH, max_length=255)

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return self.file.url


def create_thumbnails(sender, instance, created, **kwargs):
    """
    Creates the thumbnails for an image upload
    """
    for size in instance.resize_image.all():
        tn = make_thumbnail(
            instance.image.url,
            width=size.width,
            height=size.height,
        )

        if tn:
            thumb, created = Thumbnail.objects.get_or_create(
                size=size,
                upload=instance,
            )

            thumb.image = tn
            thumb.save()


def force_thumbnails(sender, instance, **kwargs):
    """
    Forces an image upload to be resaved when it doesn't have any thumbnails
    """
    if instance.id and instance.thumbnail_set.count() == 0:
        instance.save()

models.signals.post_save.connect(create_thumbnails, sender=ImageUpload)
models.signals.post_init.connect(force_thumbnails, sender=ImageUpload)



from django.template import Library
from ..models import ImageUpload

register = Library()


@register.assignment_tag
def uploader_images(q=None):
    if q:
        return ImageUpload.objects.filter(name__contains=q)
    else:
        return ImageUpload.objects.all()



import os
from django.test.testcases import TestCase
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.conf import settings


class TestUploader(TestCase):
    pass



import json
import os
import zipfile

import django
from django import forms
from django.conf import settings
from django.contrib.auth.decorators import user_passes_test
from django.core.files.uploadedfile import SimpleUploadedFile
from django.core.paginator import Paginator
from django.db.models import Q
from django.http import Http404, HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.generic import ListView

try:
    from sorl.thumbnail import get_thumbnail
except ImportError:
    from easy_thumbnails.files import get_thumbnailer

from .models import ImageUpload, FileUpload, ThumbnailSize


class LinkListView(ListView):
    use_old_output_style = False

    def get_queryset(self):
        return FileUpload.objects.all().order_by("-id")

    def get(self, request, *args, **kwargs):
        self.links = []

        for f in self.get_queryset():
            self.links.append({
                'title': f.name,
                'value': f.file.url,
            })

        if 'django.contrib.flatpages' in settings.INSTALLED_APPS:
            from django.contrib.flatpages.models import FlatPage

            for fp in FlatPage.objects.all():
                self.links.append({
                    'title': fp.title,
                    'value': fp.url,
                })

        if not self.use_old_output_style:
            return HttpResponse(
                json.dumps(self.links),
                content_type='application/json',
            )
        else:
            return super(LinkListView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        return {
            'links': self.links,
        }


class ImageListView(ListView):
    use_old_output_style = False

    def get_queryset(self):
        return ImageUpload.objects.all().order_by("-id")

    def get(self, request, *args, **kwargs):
        if not self.use_old_output_style:
            images = []

            if not getattr(settings, 'NO_IMAGE_LIST', False):
                for image in self.get_queryset():
                    images.append({
                        'title': image.name,
                        'value': image.image.url,
                    })

                    for tn in image.thumbnail_set.all():
                        images.append({
                            'title': str(tn),
                            'value': tn.image.url,
                        })

            return HttpResponse(
                json.dumps(images),
                content_type='application/json',
            )
        else:
            return super(ImageListView, self).get(request, *args, **kwargs)


@user_passes_test(lambda u: u.is_staff)
def all_files(request, template_name="uploader/all_files.html"):

    class FilesUploadForm(forms.Form):
        file = forms.FileField(label="File or Zip Archive Upload")

    class SearchForm(forms.Form):
        keywords = forms.CharField(label="Keywords", required=False)

    def add_file(filename, content, overwrite=False):
        ext = filename.rsplit(".", 1)[1].lower()
        filename = os.path.basename(filename).rsplit(".")[0]
        filename = "%s (%s)" % (filename.replace("_", ' '), ext.upper())

        try:

            if ext in ("gif", "svg", "jpg", "bmp", "jpeg", "png"):
                upload = ImageUpload(name=filename, image=content)
            else:
                upload = FileUpload(name=filename, file=content)
            upload.save()
#            if ext in ("gif", "svg", "jpg", "bmp", "jpeg", "png"):
#                upload,created = ImageUpload.objects.get_or_create(name=filename, defaults={"image": content})
#            else:
#                upload,created = FileUpload.objects.get_or_create(name=filename, defaults={"file": content})
            return upload
        except Exception, ex:
            print ex

    keywords = ""
    if request.method == "POST":
        type = request.POST.get("type")

        if "file" in request.FILES:
            form = FilesUploadForm(request.POST, request.FILES)
            if form.is_valid():
                data_file = form.cleaned_data["file"]
                first_file = None
                try:
                    if data_file.name.endswith(".zip"):
                        file_handle = data_file.open("r")
                        zip = zipfile.ZipFile(data_file)
                        for filename in zip.namelist():
                            if filename.find(".") > -1:
                                file = SimpleUploadedFile(name=filename, content=zip.read(filename))
                                upload = add_file(filename, file)
                    else:
                        raise zipfile.BadZipfile("Not a zip file")
                except zipfile.BadZipfile:
                    upload = add_file(data_file.name, data_file)
                    if upload:
                        first_file = upload
                print "First: %s, Upload: %s" % (first_file, upload)
        else:
            form = FilesUploadForm()

        search_form = SearchForm()
    else:
        type = request.GET.get("type")
        form = FilesUploadForm()

        search_form = SearchForm(request.GET)
        if search_form.is_valid():
            keywords = search_form.cleaned_data['keywords']

    try:
        ipage = int(request.GET.get('ipage', '1'))
    except ValueError:
        raise Http404

    try:
        fpage = int(request.GET.get('fpage', '1'))
    except ValueError:
        raise Http404

    if not type or type == "image":
        images = ImageUpload.objects.exclude(image="").order_by("-id")
        if keywords:
            images = images.filter(
                *[Q(name__icontains=kw) | Q(image__icontains=kw) for kw in keywords.split()]
            )
        image_paginator = Paginator(images, 21)
        image_page = image_paginator.page(ipage)
        sizes = ThumbnailSize.objects.all()

    if not type or type in ('file', 'media'):
        files = FileUpload.objects.all().order_by("-id")
        if keywords:
            files = files.filter(*[Q(name__icontains=kw) | Q(file__icontains=kw) for kw in keywords.split()])
        file_paginator = Paginator(files, 14)
        file_page = file_paginator.page(fpage)

    django_version = django.VERSION

    if (
        django_version[0] == 1 and
        django_version[1] == 8
    ):
        jquery_path = 'admin/js/jquery.js'
    else:
        jquery_path = 'admin/js/vendor/jquery/jquery.js'

    return render_to_response(
        template_name,
        locals(),
        context_instance=RequestContext(request)
    )


@user_passes_test(lambda u: u.is_staff)
def create_thumbnail(request, id, width=0, height=0):
    try:
        image = ImageUpload.objects.get(id=id)
    except ImageUpload.DoesNotExist:
        raise Http404
    if int(width) > 0 and int(height) > 0:
        try:
            thumbnail = get_thumbnail(
                image.image.name,
                '%sx%s' % (width, height),
            )
        except NameError:
            thumbnail = get_thumbnailer(image.image).get_thumbnail({
                'size': (width, height),
            })

        return HttpResponse("%s" % thumbnail.url)
    else:
        return HttpResponse("%s" % image.image.url)



import os
import urlparse
from django.conf import settings
from django.core.files.uploadedfile import InMemoryUploadedFile
from PIL import Image


def _get_thumbnail_path(path, width=None, height=None):
    """ create thumbnail path from path and required width and/or height.

        thumbnail file name is constructed like this:
            <basename>_t_[w<width>][_h<height>].<extension>
    """

    # one of width/height is required
    assert (width is not None) or (height is not None)

    basedir = os.path.dirname(path) + '/'
    base, ext = os.path.splitext(os.path.basename(path))

    # make thumbnail filename
    th_name = base + '_t'
    if (width is not None) and (height is not None):
        th_name += '_w%d_h%d' % (width, height)
    elif width is not None:
        th_name += '%d' % width  # for compatibility with admin
    elif height is not None:
        th_name += '_h%d' % height
    th_name += ext

    return urlparse.urljoin(basedir, th_name)


def _get_path_from_url(url, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ make filesystem path from url """

    if url.startswith(url_root):
        url = url[len(url_root):]  # strip media root url

    if url.startswith('/'):
        url = url[1:]

    return os.path.normpath(os.path.join(root, url))


def _get_url_from_path(path, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ make url from filesystem path """

    if path.startswith(root):
        path = path[len(root):]  # strip media root

    return urlparse.urljoin(root, path.replace('\\', '/'))


def _has_thumbnail(photo_url, width=None, height=None, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    # one of width/height is required
    assert (width is not None) or (height is not None)

    return os.path.isfile(_get_path_from_url(_get_thumbnail_path(photo_url, width, height), root, url_root))


def make_thumbnail(photo_url, width=None, height=None, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ create thumbnail """

    # one of width/height is required
    assert (width is not None) or (height is not None)

    if not photo_url:
        return None

    if isinstance(photo_url, InMemoryUploadedFile):
        temp_url = os.path.join(url_root, "temp", photo_url.name)
        th_url = _get_thumbnail_path(temp_url, width, height)
        th_path = _get_path_from_url(th_url, root, url_root)
        photo_path = photo_url  # _get_path_from_url(temp_url, root, url_root)
    else:
        temp_url = photo_url
        th_url = _get_thumbnail_path(photo_url, width, height)
        th_path = _get_path_from_url(th_url, root, url_root)
        photo_path = _get_path_from_url(photo_url, root, url_root)

    if os.access(th_path, os.R_OK):
        return th_url

    # make thumbnail

    # get original image size
    orig_w, orig_h = get_image_size(temp_url, root, url_root)
    if not isinstance(photo_url, InMemoryUploadedFile) and (orig_w is None) and (orig_h) is None:
        # something is wrong with image
        return photo_url

    # make proper size
    if (width is not None) and (height is not None):
        if (orig_w == width) and (orig_h == height):
            # same dimensions
            return None
        size = (width, height)
    elif width is not None:
        if orig_w == width:
            # same dimensions
            return None
        size = (width, orig_h)
    elif height is not None:
        if orig_h == height:
            # same dimensions
            return None
        size = (orig_w, height)

    try:
        try:
            os.makedirs(os.path.dirname(th_path))
        except Exception, ex:
            print ex
        img = Image.open(photo_path).copy()
        img.thumbnail(size, Image.ANTIALIAS)
        img.save(th_path, quality=90)
    except Exception:
        # Exception('%s, %s, %s' % (photo_path, size, th_path))
        return photo_url

    return th_url


def get_image_size(photo_url, root=settings.MEDIA_ROOT, url_root=settings.MEDIA_URL):
    """ returns image size.

        image sizes are cached (using separate locmem:/// cache instance)
    """

    path = os.path.join(root, _get_path_from_url(photo_url, root, url_root))

    try:
        size = Image.open(path).size
    except Exception, err:
        # this goes to webserver error log
        import sys
        print >>sys.stderr, '[GET IMAGE SIZE] error %s for file %r' % (err, photo_url)
        return None, None

    return size



from django.conf.urls import patterns, include, url

from .views import LinkListView, ImageListView

defaultpatterns = patterns('', (r'^uploader/', include('uploader.urls')),)

urlpatterns = patterns(
    '',
    url(r'^link_list\.js$', LinkListView.as_view()),
    url(r'^link_list-old_style\.js$', LinkListView.as_view(use_old_output_style=True)),
    url(r'^image_list\.js$', ImageListView.as_view()),
    url(r'^image_list-old_style\.js$', ImageListView.as_view(use_old_output_style=True)),
)

# for the admin
urlpatterns += patterns(
    'uploader.views',
    url(r'^all/$', "all_files", name="uploader-all"),
    url(r'^thumbnail/(\d+)/(\d+)x(\d+)/$', "create_thumbnail", name="uploader-create-thumbnail")
)
